// Fuzzy Logic Shuttle Controller
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module includes all of the front-end code for displaying the simulation and the functions that use the underlying fuzzy control module.

#region Using declarations

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

#endregion

namespace Fuzzy_Logic_Shuttle_Controller_Namespace {

	public class FuzzyLogicShuttleControl : System.Windows.Forms.Form 	{
		// This class comprises the front-end dialog box and the code that creates and uses the fuzzy control sets.

		#region Variables, initialization code, and Main

		private double[] dThrust, dThrottle, dControl;
		private double dMouseX, dMouseY;
		private ArrayList aStars;
		private ArrayList aVariableSets;
		private Random r;

		private System.Windows.Forms.PictureBox picSpace;
		private System.Windows.Forms.Timer timerRedraw;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TextBox txtControlVariable1;
		private System.Windows.Forms.TextBox txtThrottleVariable1;
		private System.Windows.Forms.TextBox txtThrustVariable1;
		private System.Windows.Forms.TextBox txtControlVariable2;
		private System.Windows.Forms.TextBox txtThrottleVariable2;
		private System.Windows.Forms.TextBox txtThrustVariable2;
		private System.Windows.Forms.TextBox txtControlVariable3;
		private System.Windows.Forms.TextBox txtThrottleVariable3;
		private System.Windows.Forms.TextBox txtThrustVariable3;
		private System.Windows.Forms.TextBox txtControlVariable4;
		private System.Windows.Forms.TextBox txtThrottleVariable4;
		private System.Windows.Forms.TextBox txtThrustVariable4;
		private System.Windows.Forms.TextBox txtThrottle4;
		private System.Windows.Forms.TextBox txtThrottle3;
		private System.Windows.Forms.TextBox txtThrottle2;
		private System.Windows.Forms.TextBox txtThrottle1;
		private System.Windows.Forms.TextBox txtThrust4;
		private System.Windows.Forms.TextBox txtThrust3;
		private System.Windows.Forms.TextBox txtThrust2;
		private System.Windows.Forms.TextBox txtThrust1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.CheckBox check1;
		private System.Windows.Forms.CheckBox check2;
		private System.Windows.Forms.CheckBox check3;
		private System.Windows.Forms.CheckBox check4;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.PictureBox picControls;

		public FuzzyLogicShuttleControl() {
			InitializeComponent();
			dThrust = new double[4];
			dThrottle = new double[4];
			dControl = new double[4];
			for (int i = 0; i < 4; i++) {
				dThrust[i] = dThrottle[i] = 0.0;
			}
			r = new Random((int) System.DateTime.Now.Ticks);
			// add stars
			aStars = new ArrayList();
			for (int i = 0; i < 100; i++)
				aStars.Add(new Star(r.Next(picSpace.Width + 10) - 5, r.Next(picSpace.Height + 10) - 5, r.Next(5) + 2, r.Next(3) + 1));
			PrepareFuzzySets();
		}

		[STAThread]
		static void Main()  {
			Application.Run(new FuzzyLogicShuttleControl());
		}

		#endregion

		#region Core functions

		public void PrepareFuzzySets() {
			FuzzyVariableSet variableSet;

			// create new array
			aVariableSets = new ArrayList();

			// set 0: basic 5x5 set
			variableSet = new FuzzyVariableSet(0.001);
			// create variables
			FuzzyVariable NB = new FuzzyVariable_Triangular("NB", -1.5, -0.5, 1.0);
			FuzzyVariable NS = new FuzzyVariable_Triangular("NS", -1.0, 0.0, 1.0);
			FuzzyVariable ZO = new FuzzyVariable_Triangular("ZO", -0.5, 0.5, 1.0);
			FuzzyVariable PS = new FuzzyVariable_Triangular("PS", 0.0, 1.0, 1.0);
			FuzzyVariable PB = new FuzzyVariable_Triangular("PB", 0.5, 1.5, 1.0);
			// add variables to fuzzy variable set
			variableSet.AddVariable(NB);
			variableSet.AddVariable(NS);
			variableSet.AddVariable(ZO);
			variableSet.AddVariable(PS);
			variableSet.AddVariable(PB);
			// use variables to create rules
			variableSet.AddRule(ZO, NB, PB);
			variableSet.AddRule(ZO, NS, PS);
			variableSet.AddRule(ZO, ZO, ZO);
			variableSet.AddRule(ZO, PS, NS);
			variableSet.AddRule(ZO, PB, NB);
			variableSet.AddRule(NB, ZO, PB);
			variableSet.AddRule(NS, ZO, PS);
			variableSet.AddRule(PS, ZO, NS);
			variableSet.AddRule(PB, ZO, NB);
			// set variable bounds
			variableSet.dMin = -1.0;
			variableSet.dMax = 1.0;
			// add to collection
			aVariableSets.Add(variableSet);

			// set 2: slightly improved 5x5 set - two additional rules
			variableSet = new FuzzyVariableSet(0.001);
			// create variables
			NB = new FuzzyVariable_Triangular("NB", -1.5, -0.5, 1.0);
			NS = new FuzzyVariable_Triangular("NS", -1.0, 0.0, 1.0);
			ZO = new FuzzyVariable_Triangular("ZO", -0.5, 0.5, 1.0);
			PS = new FuzzyVariable_Triangular("PS", 0.0, 1.0, 1.0);
			PB = new FuzzyVariable_Triangular("PB", 0.5, 1.5, 1.0);
			// add variables to fuzzy variable set
			variableSet.AddVariable(NB);
			variableSet.AddVariable(NS);
			variableSet.AddVariable(ZO);
			variableSet.AddVariable(PS);
			variableSet.AddVariable(PB);
			// use variables to create rules
			variableSet.AddRule(ZO, NB, PB);
			variableSet.AddRule(ZO, NS, PS);
			variableSet.AddRule(ZO, ZO, ZO);
			variableSet.AddRule(ZO, PS, NS);
			variableSet.AddRule(ZO, PB, NB);
			variableSet.AddRule(NB, ZO, PB);
			variableSet.AddRule(NS, ZO, PS);
			variableSet.AddRule(PS, ZO, NS);
			variableSet.AddRule(PB, ZO, NB);
			variableSet.AddRule(NS, NS, PB);  // new rule #1
			variableSet.AddRule(PS, PS, NB);  // new rule #2
			// set variable bounds
			variableSet.dMin = -1.0;
			variableSet.dMax = 1.0;
			// add to collection
			aVariableSets.Add(variableSet);

			// set 2: fully-mapped 5x5 set - 25 rules total
			variableSet = new FuzzyVariableSet(0.001);
			// create variables
			NB = new FuzzyVariable_Triangular("NB", -1.5, -0.5, 1.0);
			NS = new FuzzyVariable_Triangular("NS", -1.0, 0.0, 1.0);
			ZO = new FuzzyVariable_Triangular("ZO", -0.5, 0.5, 1.0);
			PS = new FuzzyVariable_Triangular("PS", 0.0, 1.0, 1.0);
			PB = new FuzzyVariable_Triangular("PB", 0.5, 1.5, 1.0);
			// add variables to fuzzy variable set
			variableSet.AddVariable(NB);
			variableSet.AddVariable(NS);
			variableSet.AddVariable(ZO);
			variableSet.AddVariable(PS);
			variableSet.AddVariable(PB);
			// use variables to create rules
			variableSet.AddRule(NB, NB, PB);
			variableSet.AddRule(NB, NS, PB);
			variableSet.AddRule(NB, ZO, PB);
			variableSet.AddRule(NB, PS, PS);
			variableSet.AddRule(NB, PB, ZO);
			variableSet.AddRule(NS, NB, PB);
			variableSet.AddRule(NS, NS, PB);
			variableSet.AddRule(NS, ZO, PS);
			variableSet.AddRule(NS, PS, ZO);
			variableSet.AddRule(NS, PB, NS);
			variableSet.AddRule(ZO, NB, PB);
			variableSet.AddRule(ZO, NS, PS);
			variableSet.AddRule(ZO, ZO, ZO);
			variableSet.AddRule(ZO, PS, NS);
			variableSet.AddRule(ZO, PB, NB);
			variableSet.AddRule(PS, NB, PS);
			variableSet.AddRule(PS, NS, ZO);
			variableSet.AddRule(PS, ZO, NS);
			variableSet.AddRule(PS, PS, NB);
			variableSet.AddRule(PS, PB, NB);
			variableSet.AddRule(PB, NB, ZO);
			variableSet.AddRule(PB, NS, NS);
			variableSet.AddRule(PB, ZO, NB);
			variableSet.AddRule(PB, PS, NB);
			variableSet.AddRule(PB, PB, NB);
			// set variable bounds
			variableSet.dMin = -1.0;
			variableSet.dMax = 1.0;
			// add to collection
			aVariableSets.Add(variableSet);

			// set 4: 7x7 basic array
			variableSet = new FuzzyVariableSet(0.001);
			// create variables
			NB = new FuzzyVariable_Triangular("NB", -1.5, -0.5, 1.0);
			FuzzyVariable NM = new FuzzyVariable_Triangular("NM", -1.25, -0.25, 1.0);
			NS = new FuzzyVariable_Triangular("NS", -1.0, 0.0, 1.0);
			ZO = new FuzzyVariable_Triangular("ZO", -0.5, 0.5, 1.0);
			PS = new FuzzyVariable_Triangular("PS", 0.0, 1.0, 1.0);
			FuzzyVariable PM = new FuzzyVariable_Triangular("PM", 0.25, 1.25, 1.0);
			PB = new FuzzyVariable_Triangular("PB", 0.5, 1.5, 1.0);
			// add variables to fuzzy variable set
			variableSet.AddVariable(NB);
			variableSet.AddVariable(NM);
			variableSet.AddVariable(NS);
			variableSet.AddVariable(ZO);
			variableSet.AddVariable(PS);
			variableSet.AddVariable(PM);
			variableSet.AddVariable(PB);
			// use variables to create rules
			variableSet.AddRule(ZO, NB, PB);
			variableSet.AddRule(ZO, NM, PM);
			variableSet.AddRule(ZO, NS, PS);
			variableSet.AddRule(ZO, ZO, ZO);
			variableSet.AddRule(ZO, PS, NS);
			variableSet.AddRule(ZO, PM, NM);
			variableSet.AddRule(ZO, PB, NB);
			variableSet.AddRule(NB, ZO, PB);
			variableSet.AddRule(NM, ZO, PM);
			variableSet.AddRule(NS, ZO, PS);
			variableSet.AddRule(PS, ZO, NS);
			variableSet.AddRule(PM, ZO, NM);
			variableSet.AddRule(PB, ZO, NB);
			// set variable bounds
			variableSet.dMin = -1.0;
			variableSet.dMax = 1.0;
			// add to collection
			aVariableSets.Add(variableSet);
		}

		public void ExecuteFuzzyControl() {
			// This function passes the input values to each fuzzy control function and records results.
			for (int i = 0; i < 4; i++) {
				dControl[i] = ((FuzzyVariableSet) aVariableSets[i]).FuzzyResult(dThrust[i], dThrottle[i]) / 5.0;
				dThrottle[i] = Math.Min(Math.Max(dThrottle[i] + dControl[i] / 5.0, -1.0), 1.0);
				dThrust[i] = Math.Min(Math.Max(dThrust[i] + dThrottle[i] / 5.0, -1.0), 1.0);
			}
		}

		#endregion

		#region Event handler functions

		private void picControls_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
			// This function records mouse movement within the picControls window.
			dMouseX = e.X;
			dMouseY = 1.0 - 2.0 * ((double) e.Y / (double) picControls.Height);
		}

		private void picControls_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			// This function handles button-clicks within the picControls window.
			if (dMouseX < picControls.Width / 2) {  // set thrust
				for (int i = 0; i < 4; i++)
					dThrust[i] = dMouseY;
			}
			else {  // set throttle
				for (int i =0; i < 4; i++)
					dThrottle[i] = dMouseY;
			}
		}

		private void timerRedraw_Tick(object sender, System.EventArgs e) {
			// This function contains the code that calls the fuzzy control function, updates the textboxes, updates the starfield, and redraws the viewport.

			ExecuteFuzzyControl();

			// Update textboxes
			if (check1.CheckState == CheckState.Checked) {
				txtThrottle1.Text = "" +Math.Round((decimal) dThrottle[0], 8);
				txtThrottle1.Refresh();
				txtThrust1.Text = "" + Math.Round((decimal) dThrust[0], 8);
				txtThrust1.Refresh();
				txtThrustVariable1.Text = ((FuzzyVariableSet) aVariableSets[0]).ReportCurrentVariable(dThrust[0]);
				txtThrustVariable1.Refresh();
				txtThrottleVariable1.Text = ((FuzzyVariableSet) aVariableSets[0]).ReportCurrentVariable(dThrottle[0]);
				txtThrottleVariable1.Refresh();
				txtControlVariable1.Text = ((FuzzyVariableSet) aVariableSets[0]).ReportCurrentVariable(dControl[0] * 5.0);
				txtControlVariable1.Refresh();
			}
			if (check2.CheckState == CheckState.Checked) {
				txtThrottle2.Text = "" +Math.Round((decimal) dThrottle[1], 8);
				txtThrottle2.Refresh();
				txtThrust2.Text = "" + Math.Round((decimal) dThrust[1], 8);
				txtThrust2.Refresh();
				txtThrustVariable2.Text = ((FuzzyVariableSet) aVariableSets[1]).ReportCurrentVariable(dThrust[1]);
				txtThrustVariable2.Refresh();
				txtThrottleVariable2.Text = ((FuzzyVariableSet) aVariableSets[1]).ReportCurrentVariable(dThrottle[1]);
				txtThrottleVariable2.Refresh();
				txtControlVariable2.Text = ((FuzzyVariableSet) aVariableSets[1]).ReportCurrentVariable(dControl[1] * 5.0);
				txtControlVariable2.Refresh();
			}
			if (check3.CheckState == CheckState.Checked) {
				txtThrottle3.Text = "" +Math.Round((decimal) dThrottle[2], 8);
				txtThrottle3.Refresh();
				txtThrust3.Text = "" + Math.Round((decimal) dThrust[2], 8);
				txtThrust3.Refresh();
				txtThrustVariable3.Text = ((FuzzyVariableSet) aVariableSets[2]).ReportCurrentVariable(dThrust[2]);
				txtThrustVariable3.Refresh();
				txtThrottleVariable3.Text = ((FuzzyVariableSet) aVariableSets[2]).ReportCurrentVariable(dThrottle[2]);
				txtThrottleVariable3.Refresh();
				txtControlVariable3.Text = ((FuzzyVariableSet) aVariableSets[2]).ReportCurrentVariable(dControl[2] * 5.0);
				txtControlVariable3.Refresh();
			}
			if (check4.CheckState == CheckState.Checked) {
				txtThrottle4.Text = "" +Math.Round((decimal) dThrottle[3], 8);
				txtThrottle4.Refresh();
				txtThrust4.Text = "" + Math.Round((decimal) dThrust[3], 8);
				txtThrust4.Refresh();
				txtThrustVariable4.Text = ((FuzzyVariableSet) aVariableSets[3]).ReportCurrentVariable(dThrust[3]);
				txtThrustVariable4.Refresh();
				txtThrottleVariable4.Text = ((FuzzyVariableSet) aVariableSets[3]).ReportCurrentVariable(dThrottle[3]);
				txtThrottleVariable4.Refresh();
				txtControlVariable4.Text = ((FuzzyVariableSet) aVariableSets[3]).ReportCurrentVariable(dControl[3] * 5.0);
				txtControlVariable4.Refresh();
			}

			// move stars
			for (int i = 0; i < aStars.Count;) {
				Star star = (Star) aStars[i];
				star.y += star.speed;
				if (star.y > picSpace.Height) {
					aStars.RemoveAt(i);
					aStars.Add(new Star(r.Next(picSpace.Width + 10) - 5, r.Next(10) - 10, r.Next(5) + 2, r.Next(3) + 1));
				}
				else
					i++;
			}
			picSpace.Refresh();
		}

		private void picSpace_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// This function draws the space viewport.
			int total = (check1.CheckState == CheckState.Checked ? 1 : 0) + (check2.CheckState == CheckState.Checked ? 1 : 0) + (check3.CheckState == CheckState.Checked ? 1 : 0) +(check4.CheckState == CheckState.Checked ? 1 : 0);
			int current = 0;
			Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(System.Drawing.Brushes.Black, e.ClipRectangle);
			foreach (Star star in aStars)
				g.FillEllipse(Brushes.White, star.x, star.y, star.size, star.size);
			Image image = Image.FromFile("Shuttle.png");
			if (check1.CheckState == CheckState.Checked) {
				g.DrawImage(image, picSpace.Width * (current + 1) / (total + 1) - (image.Width / 2), (int) ((picSpace.Height - image.Height - 20) * (0.5 - dThrust[0] * 0.5)));
				current++;
			}
			if (check2.CheckState == CheckState.Checked) {
				g.DrawImage(image, picSpace.Width * (current + 1) / (total + 1) - (image.Width / 2), (int) ((picSpace.Height - image.Height - 20) * (0.5 - dThrust[1] * 0.5)));
				current++;
			}
			if (check3.CheckState == CheckState.Checked) {
				g.DrawImage(image, picSpace.Width * (current + 1) / (total + 1) - (image.Width / 2), (int) ((picSpace.Height - image.Height - 20) * (0.5 - dThrust[2] * 0.5)));
				current++;
			}
			if (check4.CheckState == CheckState.Checked) {
				g.DrawImage(image, picSpace.Width * (current + 1) / (total + 1) - (image.Width / 2), (int) ((picSpace.Height - image.Height - 20) * (0.5 - dThrust[3] * 0.5)));
				current++;
			}
		}

		private void picControls_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// This function draws the control box.
			Image image = Image.FromFile("Controls.png");
			Graphics g = e.Graphics;
			g.DrawImage(image, 0, 0, picControls.Width, picControls.Height);
		}

		#endregion

		#region Uninteresting Windows Form Designer generated code

		private void InitializeComponent()	{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FuzzyLogicShuttleControl));
			this.picSpace = new System.Windows.Forms.PictureBox();
			this.timerRedraw = new System.Windows.Forms.Timer(this.components);
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.check4 = new System.Windows.Forms.CheckBox();
			this.check3 = new System.Windows.Forms.CheckBox();
			this.check2 = new System.Windows.Forms.CheckBox();
			this.check1 = new System.Windows.Forms.CheckBox();
			this.txtThrust4 = new System.Windows.Forms.TextBox();
			this.txtThrust3 = new System.Windows.Forms.TextBox();
			this.txtThrust2 = new System.Windows.Forms.TextBox();
			this.txtThrust1 = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.txtThrottle4 = new System.Windows.Forms.TextBox();
			this.txtThrottle3 = new System.Windows.Forms.TextBox();
			this.txtThrottle2 = new System.Windows.Forms.TextBox();
			this.txtThrottle1 = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.txtControlVariable4 = new System.Windows.Forms.TextBox();
			this.txtThrottleVariable4 = new System.Windows.Forms.TextBox();
			this.txtThrustVariable4 = new System.Windows.Forms.TextBox();
			this.txtControlVariable3 = new System.Windows.Forms.TextBox();
			this.txtThrottleVariable3 = new System.Windows.Forms.TextBox();
			this.txtThrustVariable3 = new System.Windows.Forms.TextBox();
			this.txtControlVariable2 = new System.Windows.Forms.TextBox();
			this.txtThrottleVariable2 = new System.Windows.Forms.TextBox();
			this.txtThrustVariable2 = new System.Windows.Forms.TextBox();
			this.label8 = new System.Windows.Forms.Label();
			this.txtControlVariable1 = new System.Windows.Forms.TextBox();
			this.txtThrottleVariable1 = new System.Windows.Forms.TextBox();
			this.txtThrustVariable1 = new System.Windows.Forms.TextBox();
			this.label7 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.picControls = new System.Windows.Forms.PictureBox();
			this.label3 = new System.Windows.Forms.Label();
			this.groupBox2.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// picSpace
			// 
			this.picSpace.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picSpace.Location = new System.Drawing.Point(8, 8);
			this.picSpace.Name = "picSpace";
			this.picSpace.Size = new System.Drawing.Size(880, 512);
			this.picSpace.TabIndex = 0;
			this.picSpace.TabStop = false;
			this.picSpace.Paint += new System.Windows.Forms.PaintEventHandler(this.picSpace_Paint);
			// 
			// timerRedraw
			// 
			this.timerRedraw.Enabled = true;
			this.timerRedraw.Interval = 40;
			this.timerRedraw.Tick += new System.EventHandler(this.timerRedraw_Tick);
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.check4);
			this.groupBox2.Controls.Add(this.check3);
			this.groupBox2.Controls.Add(this.check2);
			this.groupBox2.Controls.Add(this.check1);
			this.groupBox2.Controls.Add(this.txtThrust4);
			this.groupBox2.Controls.Add(this.txtThrust3);
			this.groupBox2.Controls.Add(this.txtThrust2);
			this.groupBox2.Controls.Add(this.txtThrust1);
			this.groupBox2.Controls.Add(this.label2);
			this.groupBox2.Controls.Add(this.txtThrottle4);
			this.groupBox2.Controls.Add(this.txtThrottle3);
			this.groupBox2.Controls.Add(this.txtThrottle2);
			this.groupBox2.Controls.Add(this.txtThrottle1);
			this.groupBox2.Controls.Add(this.label1);
			this.groupBox2.Controls.Add(this.txtControlVariable4);
			this.groupBox2.Controls.Add(this.txtThrottleVariable4);
			this.groupBox2.Controls.Add(this.txtThrustVariable4);
			this.groupBox2.Controls.Add(this.txtControlVariable3);
			this.groupBox2.Controls.Add(this.txtThrottleVariable3);
			this.groupBox2.Controls.Add(this.txtThrustVariable3);
			this.groupBox2.Controls.Add(this.txtControlVariable2);
			this.groupBox2.Controls.Add(this.txtThrottleVariable2);
			this.groupBox2.Controls.Add(this.txtThrustVariable2);
			this.groupBox2.Controls.Add(this.label8);
			this.groupBox2.Controls.Add(this.txtControlVariable1);
			this.groupBox2.Controls.Add(this.txtThrottleVariable1);
			this.groupBox2.Controls.Add(this.txtThrustVariable1);
			this.groupBox2.Controls.Add(this.label7);
			this.groupBox2.Controls.Add(this.label6);
			this.groupBox2.Controls.Add(this.label5);
			this.groupBox2.Location = new System.Drawing.Point(120, 528);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(768, 176);
			this.groupBox2.TabIndex = 2;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Fuzzy Logic Info";
			// 
			// check4
			// 
			this.check4.Location = new System.Drawing.Point(16, 144);
			this.check4.Name = "check4";
			this.check4.Size = new System.Drawing.Size(120, 24);
			this.check4.TabIndex = 34;
			this.check4.Text = "Basic 7x7 Set";
			// 
			// check3
			// 
			this.check3.Location = new System.Drawing.Point(16, 112);
			this.check3.Name = "check3";
			this.check3.Size = new System.Drawing.Size(120, 24);
			this.check3.TabIndex = 33;
			this.check3.Text = "Full 5x5 Set";
			// 
			// check2
			// 
			this.check2.Location = new System.Drawing.Point(16, 80);
			this.check2.Name = "check2";
			this.check2.Size = new System.Drawing.Size(120, 24);
			this.check2.TabIndex = 32;
			this.check2.Text = "Expanded 5x5 Set";
			// 
			// check1
			// 
			this.check1.Checked = true;
			this.check1.CheckState = System.Windows.Forms.CheckState.Checked;
			this.check1.Location = new System.Drawing.Point(16, 48);
			this.check1.Name = "check1";
			this.check1.Size = new System.Drawing.Size(120, 24);
			this.check1.TabIndex = 31;
			this.check1.Text = "Basic 5x5 Set";
			// 
			// txtThrust4
			// 
			this.txtThrust4.Location = new System.Drawing.Point(144, 144);
			this.txtThrust4.Name = "txtThrust4";
			this.txtThrust4.ReadOnly = true;
			this.txtThrust4.Size = new System.Drawing.Size(88, 20);
			this.txtThrust4.TabIndex = 26;
			this.txtThrust4.Text = "";
			this.txtThrust4.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrust3
			// 
			this.txtThrust3.Location = new System.Drawing.Point(144, 112);
			this.txtThrust3.Name = "txtThrust3";
			this.txtThrust3.ReadOnly = true;
			this.txtThrust3.Size = new System.Drawing.Size(88, 20);
			this.txtThrust3.TabIndex = 25;
			this.txtThrust3.Text = "";
			this.txtThrust3.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrust2
			// 
			this.txtThrust2.Location = new System.Drawing.Point(144, 80);
			this.txtThrust2.Name = "txtThrust2";
			this.txtThrust2.ReadOnly = true;
			this.txtThrust2.Size = new System.Drawing.Size(88, 20);
			this.txtThrust2.TabIndex = 24;
			this.txtThrust2.Text = "";
			this.txtThrust2.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrust1
			// 
			this.txtThrust1.Location = new System.Drawing.Point(144, 48);
			this.txtThrust1.Name = "txtThrust1";
			this.txtThrust1.ReadOnly = true;
			this.txtThrust1.Size = new System.Drawing.Size(88, 20);
			this.txtThrust1.TabIndex = 23;
			this.txtThrust1.Text = "";
			this.txtThrust1.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// label2
			// 
			this.label2.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label2.Location = new System.Drawing.Point(144, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(88, 16);
			this.label2.TabIndex = 22;
			this.label2.Text = "Current Thrust";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtThrottle4
			// 
			this.txtThrottle4.Location = new System.Drawing.Point(384, 144);
			this.txtThrottle4.Name = "txtThrottle4";
			this.txtThrottle4.ReadOnly = true;
			this.txtThrottle4.Size = new System.Drawing.Size(88, 20);
			this.txtThrottle4.TabIndex = 21;
			this.txtThrottle4.Text = "";
			this.txtThrottle4.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottle3
			// 
			this.txtThrottle3.Location = new System.Drawing.Point(384, 112);
			this.txtThrottle3.Name = "txtThrottle3";
			this.txtThrottle3.ReadOnly = true;
			this.txtThrottle3.Size = new System.Drawing.Size(88, 20);
			this.txtThrottle3.TabIndex = 20;
			this.txtThrottle3.Text = "";
			this.txtThrottle3.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottle2
			// 
			this.txtThrottle2.Location = new System.Drawing.Point(384, 80);
			this.txtThrottle2.Name = "txtThrottle2";
			this.txtThrottle2.ReadOnly = true;
			this.txtThrottle2.Size = new System.Drawing.Size(88, 20);
			this.txtThrottle2.TabIndex = 19;
			this.txtThrottle2.Text = "";
			this.txtThrottle2.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottle1
			// 
			this.txtThrottle1.Location = new System.Drawing.Point(384, 48);
			this.txtThrottle1.Name = "txtThrottle1";
			this.txtThrottle1.ReadOnly = true;
			this.txtThrottle1.Size = new System.Drawing.Size(88, 20);
			this.txtThrottle1.TabIndex = 18;
			this.txtThrottle1.Text = "";
			this.txtThrottle1.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// label1
			// 
			this.label1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label1.Location = new System.Drawing.Point(384, 24);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(88, 16);
			this.label1.TabIndex = 17;
			this.label1.Text = "Current Throttle";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtControlVariable4
			// 
			this.txtControlVariable4.Location = new System.Drawing.Point(624, 144);
			this.txtControlVariable4.Name = "txtControlVariable4";
			this.txtControlVariable4.ReadOnly = true;
			this.txtControlVariable4.Size = new System.Drawing.Size(136, 20);
			this.txtControlVariable4.TabIndex = 16;
			this.txtControlVariable4.Text = "";
			this.txtControlVariable4.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottleVariable4
			// 
			this.txtThrottleVariable4.Location = new System.Drawing.Point(480, 144);
			this.txtThrottleVariable4.Name = "txtThrottleVariable4";
			this.txtThrottleVariable4.ReadOnly = true;
			this.txtThrottleVariable4.Size = new System.Drawing.Size(136, 20);
			this.txtThrottleVariable4.TabIndex = 15;
			this.txtThrottleVariable4.Text = "";
			this.txtThrottleVariable4.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrustVariable4
			// 
			this.txtThrustVariable4.Location = new System.Drawing.Point(240, 144);
			this.txtThrustVariable4.Name = "txtThrustVariable4";
			this.txtThrustVariable4.ReadOnly = true;
			this.txtThrustVariable4.Size = new System.Drawing.Size(136, 20);
			this.txtThrustVariable4.TabIndex = 14;
			this.txtThrustVariable4.Text = "";
			this.txtThrustVariable4.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtControlVariable3
			// 
			this.txtControlVariable3.Location = new System.Drawing.Point(624, 112);
			this.txtControlVariable3.Name = "txtControlVariable3";
			this.txtControlVariable3.ReadOnly = true;
			this.txtControlVariable3.Size = new System.Drawing.Size(136, 20);
			this.txtControlVariable3.TabIndex = 13;
			this.txtControlVariable3.Text = "";
			this.txtControlVariable3.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottleVariable3
			// 
			this.txtThrottleVariable3.Location = new System.Drawing.Point(480, 112);
			this.txtThrottleVariable3.Name = "txtThrottleVariable3";
			this.txtThrottleVariable3.ReadOnly = true;
			this.txtThrottleVariable3.Size = new System.Drawing.Size(136, 20);
			this.txtThrottleVariable3.TabIndex = 12;
			this.txtThrottleVariable3.Text = "";
			this.txtThrottleVariable3.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrustVariable3
			// 
			this.txtThrustVariable3.Location = new System.Drawing.Point(240, 112);
			this.txtThrustVariable3.Name = "txtThrustVariable3";
			this.txtThrustVariable3.ReadOnly = true;
			this.txtThrustVariable3.Size = new System.Drawing.Size(136, 20);
			this.txtThrustVariable3.TabIndex = 11;
			this.txtThrustVariable3.Text = "";
			this.txtThrustVariable3.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtControlVariable2
			// 
			this.txtControlVariable2.Location = new System.Drawing.Point(624, 80);
			this.txtControlVariable2.Name = "txtControlVariable2";
			this.txtControlVariable2.ReadOnly = true;
			this.txtControlVariable2.Size = new System.Drawing.Size(136, 20);
			this.txtControlVariable2.TabIndex = 10;
			this.txtControlVariable2.Text = "";
			this.txtControlVariable2.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottleVariable2
			// 
			this.txtThrottleVariable2.Location = new System.Drawing.Point(480, 78);
			this.txtThrottleVariable2.Name = "txtThrottleVariable2";
			this.txtThrottleVariable2.ReadOnly = true;
			this.txtThrottleVariable2.Size = new System.Drawing.Size(136, 20);
			this.txtThrottleVariable2.TabIndex = 9;
			this.txtThrottleVariable2.Text = "";
			this.txtThrottleVariable2.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrustVariable2
			// 
			this.txtThrustVariable2.Location = new System.Drawing.Point(240, 80);
			this.txtThrustVariable2.Name = "txtThrustVariable2";
			this.txtThrustVariable2.ReadOnly = true;
			this.txtThrustVariable2.Size = new System.Drawing.Size(136, 20);
			this.txtThrustVariable2.TabIndex = 8;
			this.txtThrustVariable2.Text = "";
			this.txtThrustVariable2.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// label8
			// 
			this.label8.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label8.Location = new System.Drawing.Point(8, 24);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(128, 16);
			this.label8.TabIndex = 6;
			this.label8.Text = "Current Variable Set";
			this.label8.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// txtControlVariable1
			// 
			this.txtControlVariable1.Location = new System.Drawing.Point(624, 48);
			this.txtControlVariable1.Name = "txtControlVariable1";
			this.txtControlVariable1.ReadOnly = true;
			this.txtControlVariable1.Size = new System.Drawing.Size(136, 20);
			this.txtControlVariable1.TabIndex = 5;
			this.txtControlVariable1.Text = "";
			this.txtControlVariable1.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrottleVariable1
			// 
			this.txtThrottleVariable1.Location = new System.Drawing.Point(480, 48);
			this.txtThrottleVariable1.Name = "txtThrottleVariable1";
			this.txtThrottleVariable1.ReadOnly = true;
			this.txtThrottleVariable1.Size = new System.Drawing.Size(136, 20);
			this.txtThrottleVariable1.TabIndex = 4;
			this.txtThrottleVariable1.Text = "";
			this.txtThrottleVariable1.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// txtThrustVariable1
			// 
			this.txtThrustVariable1.Location = new System.Drawing.Point(240, 48);
			this.txtThrustVariable1.Name = "txtThrustVariable1";
			this.txtThrustVariable1.ReadOnly = true;
			this.txtThrustVariable1.Size = new System.Drawing.Size(136, 20);
			this.txtThrustVariable1.TabIndex = 3;
			this.txtThrustVariable1.Text = "";
			this.txtThrustVariable1.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// label7
			// 
			this.label7.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label7.Location = new System.Drawing.Point(624, 24);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(136, 16);
			this.label7.TabIndex = 2;
			this.label7.Text = "Current Control Variable";
			this.label7.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label6
			// 
			this.label6.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label6.Location = new System.Drawing.Point(480, 24);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(136, 16);
			this.label6.TabIndex = 1;
			this.label6.Text = "Current Throttle Variable";
			this.label6.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label5
			// 
			this.label5.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label5.Location = new System.Drawing.Point(240, 24);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(136, 16);
			this.label5.TabIndex = 0;
			this.label5.Text = "Current Thrust Variable";
			this.label5.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.picControls);
			this.groupBox1.Controls.Add(this.label3);
			this.groupBox1.Location = new System.Drawing.Point(8, 528);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(104, 176);
			this.groupBox1.TabIndex = 3;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Mission Control";
			// 
			// picControls
			// 
			this.picControls.BackColor = System.Drawing.Color.Transparent;
			this.picControls.Location = new System.Drawing.Point(8, 48);
			this.picControls.Name = "picControls";
			this.picControls.Size = new System.Drawing.Size(88, 120);
			this.picControls.TabIndex = 1;
			this.picControls.TabStop = false;
			this.picControls.Paint += new System.Windows.Forms.PaintEventHandler(this.picControls_Paint);
			this.picControls.MouseMove += new System.Windows.Forms.MouseEventHandler(this.picControls_MouseMove);
			this.picControls.MouseDown += new System.Windows.Forms.MouseEventHandler(this.picControls_MouseDown);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(8, 24);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(88, 23);
			this.label3.TabIndex = 0;
			this.label3.Text = "Thrust    Throttle";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// FuzzyLogicShuttleControl
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(894, 708);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.picSpace);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "FuzzyLogicShuttleControl";
			this.Text = "Fuzzy Logic Shuttle Controller";
			this.groupBox2.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

	}

	public class Star {
		// Obviously, his class represents a star - just a simple data structure.

		#region Variables and initialization code

		public int x, y, speed, size;
		public Star(int x, int y, int speed, int size) {
			this.x = x;
			this.y = y;
			this.speed = speed;
			this.size = size;

			#endregion

		}
	}
}
